@@ -261,9 +261,8 @@ class SimpleSAML_SessionHandlerPHP extends SimpleSAML_SessionHandler
         assert('is_string($session)');
 
         $session = unserialize($session);
-        assert('$session instanceof SimpleSAML_Session');
 
-        return $session;
+        return ($session !== false) ? $session : null;
     }
 
 
