@@ -42,7 +42,7 @@ if ($this->data['errorcode'] !== null) {
     <form action="?" method="post" name="f">
         <table>
             <tr>
-                <td rowspan="2" id="loginicon">
+                <td rowspan="2" class="loginicon">
                     <img alt=""
                         src="/<?php echo $this->data['baseurlpath']; ?>resources/icons/experience/gtk-dialog-authentication.48x48.png" />
                 </td>
@@ -152,8 +152,8 @@ if ($this->data['errorcode'] !== null) {
                 <?php
             }
             ?>
-            <tr id="regularsubmit">
-                <td></td><td></td>
+            <tr id="submit">
+                <td class="loginicon"></td><td></td>
                 <td>
                     <button class="btn"
                             onclick="this.value='<?php echo $this->t('{login:processing}'); ?>';
@@ -162,14 +162,6 @@ if ($this->data['errorcode'] !== null) {
                     </button>
                 </td>
             </tr>
-            <tr id="mobilesubmit">
-                <td></td><td></td>
-                <td>
-                    <button class="btn" tabindex="6">
-                        <?php echo $this->t('{login:login_button}'); ?>
-                    </button>
-                </td>
-            </tr>
         </table>
         <?php
         foreach ($this->data['stateparams'] as $name => $value) {
