@@ -5,7 +5,11 @@
  * filter classes direct access to the authsource ldap config
  * and connects to the ldap server.
  *
+ * Updated: 20161223 Remy Blom
+ *          - Wrapped the building of authsource config with issets
+ *
  * @author Ryan Panning <panman@traileyes.com>
+ * @author Remy Blom <remy.blom@hku.nl>
  * @package SimpleSAMLphp
  */
 abstract class sspmod_ldap_Auth_Process_BaseFilter extends SimpleSAML_Auth_ProcessingFilter {
@@ -135,20 +139,46 @@ abstract class sspmod_ldap_Auth_Process_BaseFilter extends SimpleSAML_Auth_Proce
 
 			// Build the authsource config
 			$authconfig = array();
-			$authconfig['ldap.hostname']   = @$authsource['hostname'];
-			$authconfig['ldap.enable_tls'] = @$authsource['enable_tls'];
-			$authconfig['ldap.timeout']    = @$authsource['timeout'];
-			$authconfig['ldap.debug']      = @$authsource['debug'];
-			$authconfig['ldap.basedn']     = (@$authsource['search.enable'] ? @$authsource['search.base'] : NULL);
-			$authconfig['ldap.username']   = (@$authsource['search.enable'] ? @$authsource['search.username'] : NULL);
-			$authconfig['ldap.password']   = (@$authsource['search.enable'] ? @$authsource['search.password'] : NULL);
-			$authconfig['ldap.username']   = (@$authsource['priv.read'] ? @$authsource['priv.username'] : $authconfig['ldap.username']);
-			$authconfig['ldap.password']   = (@$authsource['priv.read'] ? @$authsource['priv.password'] : $authconfig['ldap.password']);
-
-			// Only set the username attribute if the authsource specifies one attribute
-			if (@$authsource['search.enable'] && is_array(@$authsource['search.attributes'])
-			     && count($authsource['search.attributes']) == 1) {
-				$authconfig['attribute.username'] = reset($authsource['search.attributes']);
+			if (isset($authsource['hostname'])) {
+				$authconfig['ldap.hostname']   = $authsource['hostname'];
+			}
+			if (isset($authsource['enable_tls'])) {
+				$authconfig['ldap.enable_tls'] = $authsource['enable_tls'];
+			}
+			if (isset($authsource['timeout'])) {
+				$authconfig['ldap.timeout']    = $authsource['timeout'];
+			}
+			if (isset($authsource['debug'])) {
+				$authconfig['ldap.debug']      = $authsource['debug'];
+			}
+			if (isset($authsource['referrals'])) {
+				$authconfig['ldap.referrals']  = $authsource['referrals'];
+			}
+			// only set when search.enabled = true
+			if (isset($authsource['search.enable']) && $authsource['search.enable']) {
+				if (isset($authsource['search.base'])) {
+					$authconfig['ldap.basedn']     = $authsource['search.base'];
+				}
+				if (isset($authsource['search.username'])) {
+					$authconfig['ldap.username']   = $authsource['search.username'];
+				}
+				if (isset($authsource['search.password'])) {
+					$authconfig['ldap.password']   = $authsource['search.password'];
+				}
+				// Only set the username attribute if the authsource specifies one attribute
+				if (isset($authsource['search.attributes']) && is_array($authsource['search.attributes'])
+					 && count($authsource['search.attributes']) == 1) {
+					$authconfig['attribute.username'] = reset($authsource['search.attributes']);
+				}
+			}
+			// only set when priv.read = true
+			if (isset($authsource['priv.read']) && $authsource['priv.read']) {
+				if (isset($authsource['priv.username'])) {
+					$authconfig['ldap.username']   = $authsource['priv.username'];
+				}
+				if (isset($authsource['priv.password'])) {
+					$authconfig['ldap.password']   = $authsource['priv.password'];
+				}
 			}
 
 			// Merge the authsource config with the filter config,
@@ -231,6 +261,7 @@ abstract class sspmod_ldap_Auth_Process_BaseFilter extends SimpleSAML_Auth_Proce
 		$port       = $this->config->getInteger('ldap.port', 389);
 		$enable_tls = $this->config->getBoolean('ldap.enable_tls', FALSE);
 		$debug      = $this->config->getBoolean('ldap.debug', FALSE);
+		$referrals  = $this->config->getBoolean('ldap.referrals', true);
 		$timeout    = $this->config->getInteger('ldap.timeout', 0);
 		$username   = $this->config->getString('ldap.username', NULL);
 		$password   = $this->config->getString('ldap.password', NULL);
@@ -242,13 +273,14 @@ abstract class sspmod_ldap_Auth_Process_BaseFilter extends SimpleSAML_Auth_Proce
 			' Port: ' . $port .
 			' Enable TLS: ' . ($enable_tls ? 'Yes' : 'No') .
 			' Debug: ' . ($debug ? 'Yes' : 'No') .
+			' Referrals: ' . ($referrals ? 'Yes' : 'No') .
 			' Timeout: ' . $timeout .
 			' Username: ' . $username .
-			' Password: ' . str_repeat('*', strlen($password))
+			' Password: ' . (empty($password) ? '' : '********')
 		);
 
 		// Connect to the LDAP server to be queried during processing
-		$this->ldap = new SimpleSAML_Auth_LDAP($hostname, $enable_tls, $debug, $timeout, $port);
+		$this->ldap = new SimpleSAML_Auth_LDAP($hostname, $enable_tls, $debug, $timeout, $port, $referrals);
 		$this->ldap->bind($username, $password);
 
 		// All done
@@ -266,6 +298,16 @@ abstract class sspmod_ldap_Auth_Process_BaseFilter extends SimpleSAML_Auth_Proce
 	 * @return string
 	 */
 	protected function var_export($value) {
+		if (is_array($value)) {
+			// remove sensitive data
+			foreach ($value as $key => &$val) {
+				if ($key === 'ldap.password') {
+					$val = empty($val) ? '' : '********';
+				}
+			}
+			unset($val);
+		}
+
 		$export = var_export($value, TRUE);
 		$lines = explode("\n", $export);
 		foreach ($lines as &$line) {
